#    The Circle - Decentralized resource discovery software
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
   Circle daemon code

   Note about the architecture:

   The core of circle runs in the main thread
   The user interface runs in a separate thread

   The daemon is part of the core.
   The daemon contains : node, cache, file_server.
   
   name_server and chat are part of the core,
   but they do not belong to the daemon:
   they are created only when the gui is used.
   
   chat objects depend on the interface:
   there is a chat_text for the text interface, and chat_gtk for the gtk interface
   
"""

from __future__ import generators
import os, sys, threading, time, traceback, string
import math, types, socket, random, select

import __init__
import check
import error
import hash
import node
import settings
import utility
import cache
import file_server

is_jython = (str(types.IntType)[:4] == 'org.')

if is_jython:
    # No signal module.
    exit_signals = ()
else:
    import signal
    if hasattr(signal, 'SIGPWR'):
        exit_signals = (signal.SIGINT, signal.SIGTERM, signal.SIGPWR)
    else:
        exit_signals = (signal.SIGINT, signal.SIGTERM)


class Circle_daemon(utility.Task_manager):

    def status(self):
        
        self.node.lock.acquire()
        port    = self.node.address[1]
        n_peers = len(self.node.peers)
        n_bytes = self.node.network_usage
        self.node.lock.release()
                    
        self.file_server.lock.acquire()
        n_files = len(self.file_server.paths)
        self.file_server.lock.release()
        
        status = _('Circle daemon running on port %d:\n')%port
        if self.config['public_dir']:
            status += _('  Public directory: ')\
                      + self.config['public_dir']+'\n'
        else:
            status += _('  No public directory\n')
        if self.config['publish_apt']:
            status += _('  Apt cache published if available\n')
        else:
            status += _('  Apt cache not published\n')
        status += _('  %d files published (this may take a while to update)\n')%n_files\
                  + _('  Connected to %d peers, ') % n_peers\
                  + _('hashtable is %s\n') % (("inactive","active (%d links)"%len(self.node.links))[self.node.hashtable_running])\
                  + _('  Total network usage %s\n') % utility.human_size(n_bytes)
        
        if self.http_running:
            status = status + _('  Accepting %s http requests on port %d\n')\
                    % (self.http_mode,self.http_port)
        return status

    def set_http(self):

        if self.config['http'] == 0:
            str = self.stop_http()
        if self.config['http'] == 1:
            str = self.start_http(0)
            print str
        if self.config['http'] == 2:
            str = self.start_http(1)
            print str

    def stop_http(self):

        if self.http_running:
            self.http_socket.shutdown(0)
            self.http_socket.close()
            self.http_running = 0

    def start_http(self,local):
        
        if not self.http_running:            
            self.http_port = settings.default_http_port
            self.http_socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
            try:
                if local:
                    self.http_mode = 'local'
                    self.http_socket.bind(('localhost',self.http_port))
                else:
                    self.http_mode = 'remote'
                    self.http_socket.bind(('',self.http_port))
                self.http_socket.listen(5)                                    
                self.http_running = 1
                utility.Task(http_server_task,self).start()                                        
                output = _('Circle daemon will accept %s http requests on port %d\n')\
                         %(self.http_mode,self.http_port)
                output += _('Check http://localhost:%d/ in your web browser\n')\
                          %(self.http_port)
            except:
                output = _('http socket already in use')
        else:
            output = _('Circle daemon accepting http requests on port %d\n')\
                         % self.http_port
            
        return output


    def run(self, initial_command = None):
        """
        This is the daemon mainloop.
        """
        if node.need_proxy():
            raise error.Error(_("Circle daemon can not be run on a NAT network."))

        if not os.fork():
            #os.setsid()
            #sys.stdin.close()
            #dev_null = open('/dev/null','wt')
            #sys.stdout.close()
            #sys.stderr.close()
            #sys.stdout = dev_null
            #sys.stderr = dev_null

            com_sock = socket.socket(socket.AF_UNIX,socket.SOCK_STREAM)
            com_sock_file = os.path.join(utility.config_dir,'daemon_socket')

            try:
                os.unlink(com_sock_file) 
            except OSError:
                pass
            
            com_sock.bind(com_sock_file)
            com_sock.listen(5)
        
            self.start()
            self.node = node.Node()
            self.node.start()

            self.file_server = file_server.File_server(self.node)
            self.cache = cache.Cache(self.node)            

            self.config = utility.get_config('daemon', { })
            for pair in [('public_dir', ''),
                         ('private_dir', ''),
                         ('download_dir', ''),
                         ('stay_alive', 0),
                         ('http', 0),
                         ('publish_apt', 0),
                         ('keep_sharing', 0)]:

                if not self.config.has_key(pair[0]):
                    self.config[pair[0]] = pair[1]

            self.file_server.set_roots(self.config)            
            self.file_server.start()
            self.cache.start()

            #def signal_handler(signal, frame): 
            #    print _("Received signal %d, exiting.") % signal
            #    self.stopping = 1

            def signal_handler(signal, frame):
                raise error.Error('signal')
            for s in exit_signals:
                signal.signal(s, signal_handler)

            self.stopping = 0
            self.interface_running = 0
            self.http_running = 0
            self.set_http()
                
            utility.threadnice_mainloop_start()

            while not self.stopping:

                # at this point it cannot be 'text'
                if initial_command:
                    command = initial_command
                    initial_command = None
                    #if command == 'text\n':                        
                    input = sys.stdin
                    output = sys.stdout                        
                    connection = None
                else:
                    while not select.select([com_sock],[],[],0.1)[0]:
                        if self.stopping: break
                        try:                            
                            utility.threadnice_mainloop_iteration()
                        except:
                            traceback.print_exc()
                            
                    #if stopping we need to exit the 2nd loop too:
                    if self.stopping: break
                    
                    connection = com_sock.accept()[0]
                    input  = connection.makefile("rt")
                    output = connection.makefile("wt")
                    command = input.readline()

                if command == 'quit\n':
                    if self.interface_running:
                        output.write('Shutting down interface...\n')
                        self.interface.shutdown()
                        self.interface_running = 0
                    self.stopping = 1
                    break

                if command == 'shutdown\n':
                    if self.interface_running:
                        output.write('Shutting down interface...')
                        self.interface.shutdown()
                        self.interface_running = 0
                    else:
                        output.write('No interface running.')

                elif command == 'sync config\n':
                    self.config = utility.get_config('daemon',{})
                    self.file_server.set_roots(config)
                    output.write(_("Active circle daemon has updated its settings."))

                elif command == 'status\n':
                    str = self.status()
                    output.write(str)

                elif command == 'activate\n':
                    self.node.activate_hashtable()
                    output.write("Hashtable activated")

                elif command == 'gtk\n':
                    import gtk
                    import circle_gtk
                    import settings

                    def gui_task(self):
                        # after importing gtk the terminal encoding needs to be updated
                        settings.terminal_encoding = sys.getdefaultencoding()
                        global _gtkthread
                        _gtkthread = threading.currentThread()

                        # reload the roots
                        if not self.config['keep_sharing']:                                    
                            self.file_server.set_roots(self.config)

                        #this causes a gtk warning on the 2nd time
                        gtk.threads_init()                                
                        gtk.threads_enter()
                        self.interface = circle_gtk.Circle_gtk(self)
                        self.interface.run_main()
                        gtk.threads_leave()
                        #del circle_gtk
                        self.interface_running=0
                        #del gtk
                        if not self.config['stay_alive']:
                            output.write(_('Stopping Circle daemon...\n'))
                            self.stopping = 1
                        if not self.config['keep_sharing']:
                            output.write(_('Daemon will not share files.\n'))
                            self.file_server.roots = [ ]                                    

                    if not self.interface_running:
                        self.interface_running = 1
                        utility.Task(gui_task,self).start()
                        output.write('Starting gtk interface')
                    else:
                        output.write("Circle interface already running")

                elif command == 'text\n':

                    def text_task(self, input, output):
                        import circle_text
                        import settings
                        settings.terminal_encoding = sys.getdefaultencoding()
                        self.interface = circle_text.Circle_text(self,input,output)
                        self.interface.run_main()
                        self.interface_running = 0
                        #try:
                        input.close()
                        if connection:
                            connection.close()
                        output.close()
                        #except:
                        #    pass

                    if not self.interface_running:
                        self.interface_running = 1
                        output.write('Starting text interface')
                        utility.Task(text_task,self,input,output).start()
                    else:
                        output.write("Circle interface already running")                
                        input.close()
                        if connection:
                            connection.close()
                        output.close()

                elif command == 'debug\n':
                    for item in threading.enumerate():
                        output.write(repr(item)+'\n\n')

                elif command[0:6] == 'search':
                    utility.Task(
                        search_task,self,command[7:],input,output,connection).start()

                elif command[0:4] == 'find':
                    utility.Task(
                        find_task,self,command[5:],input,output,connection).start()

                elif command[0:16] == 'get circle-file:':
                    utility.Task(
                        get_file_task,self,command[16:38],input,output,connection).start()

                elif command == 'http local\n':
                    self.config['http'] = 2
                    output.write(self.start_http(1))

                elif command == 'http remote\n':
                    self.config['http'] = 1
                    output.write(self.start_http(0))

                elif command == 'http stop\n':
                    self.config['http'] = 0
                    self.stop_http()
                else:
                    output.write(_('Huh?'))

                if command.split()[0] not in ['search','find','get','text']:
                    try:
                        input.close()
                        connection.close()
                        output.close()
                    except:
                        pass
                    
            utility.threadnice_mainloop_stop()
            #except error.Error: # signal, eg ^C, SIGTERM
            #    pass
                
            for s in exit_signals:
                signal.signal(s, signal.SIG_IGN)

            self.cache.stop()
            self.file_server.stop()

            if self.node.hashtable_running:
                initial_links = len(self.node.links)
                self.node.deactivate_hashtable()
                while self.node.hashtable_offloaders or self.node.links:
                    utility.threadnice_mainloop_iteration()
                    time.sleep(0.01)
                    output.write("offloading : %.1f%% \r" % 
                        ((initial_links-len(self.node.links))*100.0 /
                        initial_links ))
                    output.flush()
                
            self.node.stop()

            com_sock.close()
            os.unlink(com_sock_file)
            
            if self.stopping:
                self.running=0
                try:
                    output.write(_('Circle daemon stopped.'))
                    input.close()
                    output.flush()
                    output.close()
                    connection.close()
                    self.stop_http()
                except:
                    print _('Circle daemon stopped.')

            self.stop()
        else:
            print _("Circle daemon started.")


def http_server_task(daemon):

    while daemon.running:
        try:
            (clientsocket, address) = daemon.http_socket.accept()
        except:
            break
        output = clientsocket.makefile("wt")
        input = clientsocket.makefile("rt")
        line = input.readline()
        try:
            request = line.split()[1]
        except:
            print "could not understand request by %s"%utility.hostname_of_ip(address[0])
            continue

        strtime = time.asctime()
        if request[1:13]=='circle-find/':
            utility.Task(find_task,daemon,request[13:],input,output,clientsocket,1).start()
            print "Request: people matching \"%s\" by %s, on %s" \
                  %(request[13:],utility.hostname_of_ip(address[0]),strtime)
        elif request[1:15]=='circle-search/':
            utility.Task(search_task,daemon,request[15:],input,output,clientsocket,1).start()
            print "Request: files matching \"%s\" by %s, on %s" \
                  %(request[15:],utility.hostname_of_ip(address[0]),strtime)
        elif request[1:13]=='circle-file/':
            utility.Task(get_file_task,daemon,request[13:35],input,output,clientsocket,1).start()
        else:
            utility.Task(search_task,daemon,'',input,output,clientsocket,1).start()
 
    daemon.http_socket.close()



def stress(str,key):
    if key.__len__() == 0:
        return str
    try:
        i=str.lower().find(key)
    except:
        return ""
    if i!=-1: 
        return str[0:i]+"<b>"+str[i:i+key.__len__()]+"</b>"+stress(str[i+key.__len__():],key)
    else:
        return str


def url(ref,circle_url,filename,port):
    return "<a href =\"http://localhost:%d/circle-file/%s/%s\">%s</a>"\
           %(port,circle_url[12:],filename,ref)


def search_task(daemon,query,input,output,connection,html=0):

    if html:
        output.write('HTTP/1.1 200 OK\n')
        output.write('content-type: text/html\n')
        output.write('Cache-control: max-age=60\n')
        output.write('\n')
        file=open(utility.find_file("search.html"))
        str=file.read()
        file.close()    
        if daemon.http_mode=='local':
            image_tag = "<img src=\"file://"+utility.find_file("circle-logo.png")\
                        +"\" title=\"\" alt=\"\" border=0 style=\"width: 150px; height: 50px;\"> "
            hostname = 'localhost'
        else:
            image_tag = "<img src=\"http://thecircle.org.au/circle-logo.png\" "\
                        +" title=\"\" alt=\"\" border=0 style=\"width: 150px; height: 50px;\"> "
            hostname = 'thecircle.dyndns.org'
        output.write(str%(hostname,daemon.http_port,hostname,daemon.http_port,image_tag))
        output.flush()

    for char in '+-_.,?()![]':
        query = query.replace(char," ")
    query=query.lower()
    list=query.split()
    if list:
        key=list[0]
    else:
        key=''

    if key.__len__()<3:
        if html:
            output.write("<p>Keyword %s too short: must be at least 3 characters<p></body></html>"%key)
        else:
            output.write("Keyword %s too short: must be at least 3 characters"%key)            
        input.close()
        output.close()
        connection.close()
        return    
        
    pipe = daemon.node.retrieve(hash.hash_of(key))
    results = []
    restricted = 0
    while not pipe.finished() and not restricted:
        for item in pipe.read_all():

            if results.__len__()==100:
                restricted = 1
                break
            
            if item[1]['name'] not in results:
                results.append(item[1]['name'])
                filename = utility.force_string(item[1]['filename'])
                extension = string.split(string.split(filename,'.')[-1],'-')[0]
                lext = string.lower(extension)
                if lext in ['mp3','ogg']:
                    music=1
                else:
                    music=0
                if item[1].has_key('music_title'):
                    ref = utility.force_string(item[1]['music_title'])
                    if ref.strip()=='':
                        ref= filename
                else:
                    ref = utility.force_string(item[1]['filename'])

                if not html:
                    output.write(hash.hash_to_url(item[1]['name'])+" \t"\
                                 +utility.human_size(item[1]['length'])+" \t"+filename+'\n')
                else:
                    output.write("<p class=g><t>\n")
                    output.write(url(stress(ref,key),\
                                     hash.hash_to_url(item[1]['name']),filename,daemon.http_port))
                    output.write("<br><font size=-1>")

                    if music:
                        line=0
                        if item[1].has_key('music_artist'):
                            artist_str="Artist:"+stress(utility.force_string(item[1]['music_artist']),key)
                            output.write(artist_str)
                            line=1
                        if item[1].has_key('music_album'):
                            if item[1]['music_album']!='':
                                album_str=" Album:"+stress(utility.force_string(item[1]['music_album']),key)
                                output.write(album_str)
                                line=1
                        if line:
                            output.write("<br>")
                
                    #line=0
                    #if item[1].has_key('rate'):
                    #    output.write("Rate:"+item[1]['rate']+".")
                    #    line=1
                    #if item[1].has_key('freq'):
                    #    output.write("   Frequency:"+item[1]['freq']+".")
                    #    line=1
                    #if item[1].has_key('misc'):
                    #    output.write("   --  "+item[1]['misc'])
                    #    line=1
                    #if line:
                    #    output.write("<br>")

                    output.write("File name: "+stress(filename,key))
                    output.write("<br>")
                    if item[1].has_key('mime'):
                        if item[1]['mime'] != 'unknown':
                            output.write('Mime: '+item[1]['mime'].replace(' \x08',''))
                            output.write("<br>")
                    
                    if item[1].has_key('generic'):
                        output.write(item[1]['generic'].replace('\x08',''))
                        output.write("<br>")
                    else:
                        for i in item[1].items():
                            if i[0] not in ['keywords',
                                            'mime','length',
                                            'name','filename',
                                            'type','music_album',
                                            'music_artist','music_title']:
                                output.write(i[0]+':'+stress(i[1],key)+'  ')
                                output.write("<br>")

                    output.write("<font color=#008000>"+hash.hash_to_url(item[1]['name'])+" -  "\
                                 +utility.human_size(item[1]['length'])+"- </font>")

                    #output.write("<font color=#656565>Sources:<a href="">Sources</a></font>")
                    output.write("</font></p>\n")

        time.sleep(0.5)
        try:
            output.flush()
        except:
            return
    if not results:
        try:
            if html:
                output.write("<br><br><p>Your search: <b>"+key+"</b> did not match any document.</p>")
            else:
                output.write("No document matching \""+key+"\"")
          
        except:
            pass
    else:
        if html:
            if restricted:
                output.write("<p><br>Displaying only 100 results for <b>%s</b></p>" % key)
            else:
                output.write("<p><br>Found %d files matching <b>%s</b></p>" % (results.__len__(),key))
            output.write("<hr><p class=g align=\"right\"></font size=-1>"\
                         +utility.force_string(random.choice(settings.gratuitous_plugs))+"</font></p><hr>")
    if html:
        output.write("</body></html>")
    pipe.stop()
    try:
        input.close()
        output.close()
        connection.close()
    except:
        pass
    #print "returned %d files for \"%s\""%(results.__len__(), query)
    sys.stdout.flush()


def find_task(daemon,query,input,output,connection,html=0):

    import safe_pickle

    if html:
        output.write('HTTP/1.1 200 OK\n')
        output.write('content-type: text/html\n')
        output.write('Cache-control: max-age=60\n')
        output.write('\n')

        file=open(utility.find_file("search.html"))
        str=file.read()
        file.close()

        if daemon.http_mode=='local':
            image_tag = "<img src=\"file://"+utility.find_file("circle-logo.png")\
                        +"\" title=\"\" alt=\"\" border=0 style=\"width: 150px; height: 50px;\"> "
            hostname = 'localhost'
        else:
            image_tag = "<img src=\"http://thecircle.org.au/circle-logo.png\" "\
                        +" title=\"\" alt=\"\" border=0 style=\"width: 150px; height: 50px;\"> "
            hostname = 'thecircle.dyndns.org'
    
        output.write(str%(hostname,daemon.http_port,hostname,daemon.http_port,image_tag))
        output.flush()
    
    for char in '+-_.,?()![]':
        query = query.replace(char," ")
    query=query.lower()
    list=query.split()
    if list:
        key=list[0]
        pipe = daemon.node.retrieve(hash.hash_of('identity-name '+key), settings.identity_redundancy)
    else:
        pipe = daemon.node.retrieve(hash.hash_of('service identity'), settings.identity_redundancy)
        
    results = []
    while not pipe.finished():

        list = pipe.read_all()
        prev_pair = None
        for pair in list:
            if pair == prev_pair:
                continue
            link, item = prev_pair = pair

            try:
                item = utility.check_and_demangle_item(item)
            except:
                continue

            if item['key'] not in results:
                results.append(item['key'])
                name = hash.hash_of(safe_pickle.dumps(item['key']))                
                check.check_is_name(name)
                str = "circle-person:"+string.join(map(lambda a:hex(ord(a))[2:], name), '')

                if html:
                    output.write("<p class=g><t><b>\n")
                    output.write(item['name'])
                    output.write(" ("+utility.force_string(item['human-name'])+")")
                    output.write("</b><br>")
                    if item['description']:
                        output.write(
                            "<font size=-1>"+utility.force_string(item['description'])+"<br></font>")
                    output.write("<font color=#008000 size=-1> "+str+"</font></p>\n")
                else:
                    output.write(
                        item['name']+" ("+utility.force_string(item['human-name'])+") \t"+str+'\n')
                
        time.sleep(0.5)
        try:
            output.flush()
        except:
            return
        
    if not results:
        try:
            if html:
                output.write("<br><br><p>Your search: <b>"+key+"</b> did not match any circle user.</p>")
            else:
                output.write("No user matching \""+key+"\"")
        except:
            pass
    else:
        if results.__len__()==1:
            what = "one identity"
        else:
            what = "%d identities"%results.__len__()
        if query:
            msg = "Returned " + what + " matching <b>\"" + query + "\"</b>."
        else:
            msg = "Returned " + what +"."
        if html:
            output.write("<p><br>"+msg+"</p>")
            output.write("<hr><p class=g align=\"right\"></font size=-1>"\
                         +utility.force_string(random.choice(settings.gratuitous_plugs))+"</font></p><hr>")
            
    if html:
        output.write("</body></html>")
    pipe.stop()
    try:
        input.close()
        output.close()
        connection.close()
    except:
        #connection reset by peer...
        pass
    #print "returned %d people for \"%s\""%(results.__len__(), query)
    sys.stdout.flush()


def mime_header(fname):
    # we determine the type from extension
    # we should rather use the 'mime' field of data,
    # but it seems to be broken for the moment...

    extension = string.split(string.split(fname,'.')[-1],'-')[0]
    lext = string.lower(extension)
    if lext in ['html','htm']:
        return "Content-type: text/html\n"
    if lext == 'pdf':
        return "content-type: application/pdf\n"
    if lext == 'ps':
        return "content-type: application/postscript\n"
    elif lext == 'mp3':
        return "content-type: audio/mp3\n"
    elif lext== 'ogg':
        return "content-type: audio/ogg\n"
    elif lext== 'png':
        return "content-type: image/png\n"
    elif lext in ['jpg','jpeg']:
        return "content-type: image/jpeg\n"
    elif lext== 'gif':
        return "content-type: image/gif\n"
    elif lext== 'xbm':
        return "content-type: image/xbm\n"
    elif lext== 'bmp':
        return "content-type: image/bmp\n"
    elif lext in ['mpg','mpeg']:
        return "content-type: video/mpeg\n"
    elif lext== 'avi':
        return "content-type: video/avi\n"
    else:
        return ""



def get_file_task(daemon,url,input,output,connection,html=0):
    import file_server
    try:
        name=hash.url_to_hash("circle-file:"+url)    
        salt_name=name+daemon.node.salt
    except:
        output.write(_('error: Not a circle URL.\n')+url)
        input.close()
        output.close()
        connection.close()
        return

    if daemon.node.data.has_key(salt_name):
        
        data=daemon.node.data.get(salt_name)[0]
        filename=daemon.file_server.paths.get(name)[0]

        if html:
            output.write('HTTP/1.1 200 OK\n')
            output.write(mime_header(filename))
            output.write('Content-length: %d'%data['length']+'\n')
            output.write('\n')
            output.flush()
            try:
                file = open(filename,'r')
                while 1:
                    chunk=file.read(1024*1024)
                    if chunk=='':
                        break
                    output.write(chunk)
                    output.flush()
                file.close()
            except:
                output.write(_('file access error.\n'))
        else:
            output.write('File already in your directory: '+filename)
                
    else:
        pipe = daemon.node.retrieve(name)
        list = pipe.read_until_finished()
        if not list:
            if html:
                output.write(_('<html><head></head><body>Could not locate file.\n</body></html>'))
            else:
                output.write(_('Could not locate file.\n'))
                
            input.close()
            output.close()
            connection.close()
            return
        data = list[0][1]
        if html:
            output.write('HTTP/1.1 200 OK\n')
            output.write(mime_header(data['filename']))
            output.write('Content-length: %d'%data['length']+'\n')
            output.write('\n')
            output.flush()

        links = [ ]
        for item in list:
            links.append(item[0])

        if daemon.config['public_dir']:
            downloader = file_server.Circle_Downloader(daemon.node, data,links,daemon.config['public_dir'])
            daemon.file_server.downloaders.append(downloader)
            if html:
                downloader.files.append(output)  #downloader will write to it and close it...
            downloader.start()

            if not html:
                while downloader.running:
                    output.write(downloader.basename + (" : %.1f%% \r" % (
                        (downloader.get_bytes_downloaded()+1)*100.0 /
                        (downloader.get_bytes_total()+1) )))
                    try:
                        output.flush()
                    except:
                        downloader.stop()
                    time.sleep(0.3)
                output.write(downloader.basename + (" : %.1f%% \r" % (
                    (downloader.get_bytes_downloaded()+1)*100.0 /
                    (downloader.get_bytes_total()+1) )))             
                output.close()
                
            else:
                while downloader.running:
                    try:
                        output.flush()
                    except:
                        downloader.stop()
                    time.sleep(0.3)

            if downloader.success:
                daemon.node.publish(name,data)
                daemon.file_server.paths[name] = (downloader.filename,os.path.getmtime(downloader.filename))
        else:
            output.write(_(' You need to define a public directory'\
                           +' where the Circle daemon will download files\n'
                           +' Type \'circle publish <directory>\' in a terminal\n'))
    try:
        output.close()
        input.close()
        connection.close()
    except:
        pass
